@@ -8,6 +8,8 @@ from . import LcnDevice
 from .const import CONF_CONNECTIONS, CONF_MOTOR, CONF_REVERSE_TIME, DATA_LCN
 from .helpers import get_connection
 
+PARALLEL_UPDATES = 0
+
 
 async def async_setup_platform(
     hass, hass_config, async_add_entities, discovery_info=None
@@ -49,9 +51,10 @@ class LcnOutputsCover(LcnDevice, CoverEntity):
             ]
         else:
             self.reverse_time = None
-        self._closed = None
-        self.state_up = False
-        self.state_down = False
+
+        self._is_closed = False
+        self._is_closing = False
+        self._is_opening = False
 
     async def async_added_to_hass(self):
         """Run when entity about to be added to hass."""
@@ -66,28 +69,53 @@ class LcnOutputsCover(LcnDevice, CoverEntity):
     @property
     def is_closed(self):
         """Return if the cover is closed."""
-        return self._closed
+        return self._is_closed
+
+    @property
+    def is_opening(self):
+        """Return if the cover is opening or not."""
+        return self._is_opening
+
+    @property
+    def is_closing(self):
+        """Return if the cover is closing or not."""
+        return self._is_closing
+
+    @property
+    def assumed_state(self):
+        """Return True if unable to access real state of the entity."""
+        return True
 
     async def async_close_cover(self, **kwargs):
         """Close the cover."""
-        self._closed = True
-
         state = pypck.lcn_defs.MotorStateModifier.DOWN
-        self.address_connection.control_motors_outputs(state)
+        if not await self.address_connection.control_motors_outputs(
+            state, self.reverse_time
+        ):
+            return
+        self._is_opening = False
+        self._is_closing = True
         self.async_write_ha_state()
 
     async def async_open_cover(self, **kwargs):
         """Open the cover."""
-        self._closed = False
         state = pypck.lcn_defs.MotorStateModifier.UP
-        self.address_connection.control_motors_outputs(state, self.reverse_time)
+        if not await self.address_connection.control_motors_outputs(
+            state, self.reverse_time
+        ):
+            return
+        self._is_closed = False
+        self._is_opening = True
+        self._is_closing = False
         self.async_write_ha_state()
 
     async def async_stop_cover(self, **kwargs):
         """Stop the cover."""
-        self._closed = None
         state = pypck.lcn_defs.MotorStateModifier.STOP
-        self.address_connection.control_motors_outputs(state, self.reverse_time)
+        if not await self.address_connection.control_motors_outputs(state):
+            return
+        self._is_closing = False
+        self._is_opening = False
         self.async_write_ha_state()
 
     def input_received(self, input_obj):
@@ -98,15 +126,19 @@ class LcnOutputsCover(LcnDevice, CoverEntity):
         ):
             return
 
-        if input_obj.get_output_id() == self.output_ids[0]:
-            self.state_up = input_obj.get_percent() > 0
-        else:  # self.output_ids[1]
-            self.state_down = input_obj.get_percent() > 0
-
-        if self.state_up and not self.state_down:
-            self._closed = False  # Cover open
-        elif self.state_down and not self.state_up:
-            self._closed = True  # Cover closed
+        if input_obj.get_percent() > 0:  # motor is on
+            if input_obj.get_output_id() == self.output_ids[0]:
+                self._is_opening = True
+                self._is_closing = False
+            else:  # self.output_ids[1]
+                self._is_opening = False
+                self._is_closing = True
+            self._is_closed = self._is_closing
+        else:  # motor is off
+            # cover is assumed to be closed if we were in closing state before
+            self._is_closed = self._is_closing
+            self._is_closing = False
+            self._is_opening = False
 
         self.async_write_ha_state()
 
@@ -122,7 +154,9 @@ class LcnRelayCover(LcnDevice, CoverEntity):
         self.motor_port_onoff = self.motor.value * 2
         self.motor_port_updown = self.motor_port_onoff + 1
 
-        self._closed = None
+        self._is_closed = False
+        self._is_closing = False
+        self._is_opening = False
 
     async def async_added_to_hass(self):
         """Run when entity about to be added to hass."""
@@ -132,30 +166,52 @@ class LcnRelayCover(LcnDevice, CoverEntity):
     @property
     def is_closed(self):
         """Return if the cover is closed."""
-        return self._closed
+        return self._is_closed
+
+    @property
+    def is_opening(self):
+        """Return if the cover is opening or not."""
+        return self._is_opening
+
+    @property
+    def is_closing(self):
+        """Return if the cover is closing or not."""
+        return self._is_closing
+
+    @property
+    def assumed_state(self):
+        """Return True if unable to access real state of the entity."""
+        return True
 
     async def async_close_cover(self, **kwargs):
         """Close the cover."""
-        self._closed = True
         states = [pypck.lcn_defs.MotorStateModifier.NOCHANGE] * 4
         states[self.motor.value] = pypck.lcn_defs.MotorStateModifier.DOWN
-        self.address_connection.control_motors_relays(states)
+        if not await self.address_connection.control_motors_relays(states):
+            return
+        self._is_opening = False
+        self._is_closing = True
         self.async_write_ha_state()
 
     async def async_open_cover(self, **kwargs):
         """Open the cover."""
-        self._closed = False
         states = [pypck.lcn_defs.MotorStateModifier.NOCHANGE] * 4
         states[self.motor.value] = pypck.lcn_defs.MotorStateModifier.UP
-        self.address_connection.control_motors_relays(states)
+        if not await self.address_connection.control_motors_relays(states):
+            return
+        self._is_closed = False
+        self._is_opening = True
+        self._is_closing = False
         self.async_write_ha_state()
 
     async def async_stop_cover(self, **kwargs):
         """Stop the cover."""
-        self._closed = None
         states = [pypck.lcn_defs.MotorStateModifier.NOCHANGE] * 4
         states[self.motor.value] = pypck.lcn_defs.MotorStateModifier.STOP
-        self.address_connection.control_motors_relays(states)
+        if not await self.address_connection.control_motors_relays(states):
+            return
+        self._is_closing = False
+        self._is_opening = False
         self.async_write_ha_state()
 
     def input_received(self, input_obj):
@@ -165,6 +221,11 @@ class LcnRelayCover(LcnDevice, CoverEntity):
 
         states = input_obj.states  # list of boolean values (relay on/off)
         if states[self.motor_port_onoff]:  # motor is on
-            self._closed = states[self.motor_port_updown]  # set direction
+            self._is_opening = not states[self.motor_port_updown]  # set direction
+            self._is_closing = states[self.motor_port_updown]  # set direction
+        else:  # motor is off
+            self._is_opening = False
+            self._is_closing = False
+            self._is_closed = states[self.motor_port_updown]
 
         self.async_write_ha_state()
