@@ -145,6 +145,7 @@ PLATFORMS = [
     "fan",
     "light",
     "lock",
+    "scene",
     "sensor",
     "switch",
     "vacuum",
@@ -190,7 +191,7 @@ def embedded_broker_deprecated(value):
 CONFIG_SCHEMA = vol.Schema(
     {
         DOMAIN: vol.All(
-            cv.deprecated(CONF_TLS_VERSION, invalidation_version="0.115"),
+            cv.deprecated(CONF_TLS_VERSION),
             vol.Schema(
                 {
                     vol.Optional(CONF_CLIENT_ID): cv.string,
@@ -944,7 +945,9 @@ class MQTT:
                 )
 
             birth_message = Message(**self.conf[CONF_BIRTH_MESSAGE])
-            self.hass.loop.create_task(publish_birth_message(birth_message))
+            asyncio.run_coroutine_threadsafe(
+                publish_birth_message(birth_message), self.hass.loop
+            )
 
     def _mqtt_on_message(self, _mqttc, _userdata, msg) -> None:
         """Message received callback."""
@@ -1248,7 +1251,7 @@ async def cleanup_device_registry(hass, device_id):
     if (
         device_id
         and not hass.helpers.entity_registry.async_entries_for_device(
-            entity_registry, device_id
+            entity_registry, device_id, include_disabled_entities=True
         )
         and not await device_trigger.async_get_triggers(hass, device_id)
         and not tag.async_has_tags(hass, device_id)
@@ -1485,20 +1488,22 @@ def async_subscribe_connection_status(hass, connection_status_callback):
 
     connection_status_callback_job = HassJob(connection_status_callback)
 
-    @callback
-    def connected():
-        hass.async_add_hass_job(connection_status_callback_job, True)
+    async def connected():
+        task = hass.async_run_hass_job(connection_status_callback_job, True)
+        if task:
+            await task
 
-    @callback
-    def disconnected():
-        _LOGGER.error("Calling connection_status_callback, False")
-        hass.async_add_hass_job(connection_status_callback_job, False)
+    async def disconnected():
+        task = hass.async_run_hass_job(connection_status_callback_job, False)
+        if task:
+            await task
 
     subscriptions = {
         "connect": async_dispatcher_connect(hass, MQTT_CONNECTED, connected),
         "disconnect": async_dispatcher_connect(hass, MQTT_DISCONNECTED, disconnected),
     }
 
+    @callback
     def unsubscribe():
         subscriptions["connect"]()
         subscriptions["disconnect"]()
