@@ -1,20 +1,23 @@
 """This platform provides support for sensor data from RainMachine."""
-import logging
+from functools import partial
+from typing import Callable
 
-from homeassistant.core import callback
-from homeassistant.helpers.dispatcher import async_dispatcher_connect
+from regenmaschine.controller import Controller
+
+from homeassistant.config_entries import ConfigEntry
+from homeassistant.const import TEMP_CELSIUS, VOLUME_CUBIC_METERS
+from homeassistant.core import HomeAssistant, callback
+from homeassistant.helpers.update_coordinator import DataUpdateCoordinator
 
 from . import RainMachineEntity
 from .const import (
-    DATA_CLIENT,
+    DATA_CONTROLLER,
+    DATA_COORDINATOR,
     DATA_PROVISION_SETTINGS,
     DATA_RESTRICTIONS_UNIVERSAL,
-    DOMAIN as RAINMACHINE_DOMAIN,
-    SENSOR_UPDATE_TOPIC,
+    DOMAIN,
 )
 
-_LOGGER = logging.getLogger(__name__)
-
 TYPE_FLOW_SENSOR_CLICK_M3 = "flow_sensor_clicks_cubic_meter"
 TYPE_FLOW_SENSOR_CONSUMED_LITERS = "flow_sensor_consumed_liters"
 TYPE_FLOW_SENSOR_START_INDEX = "flow_sensor_start_index"
@@ -25,7 +28,7 @@ SENSORS = {
     TYPE_FLOW_SENSOR_CLICK_M3: (
         "Flow Sensor Clicks",
         "mdi:water-pump",
-        "clicks/m^3",
+        f"clicks/{VOLUME_CUBIC_METERS}",
         None,
         False,
         DATA_PROVISION_SETTINGS,
@@ -57,7 +60,7 @@ SENSORS = {
     TYPE_FREEZE_TEMP: (
         "Freeze Protect Temperature",
         "mdi:thermometer",
-        "°C",
+        TEMP_CELSIUS,
         "temperature",
         True,
         DATA_RESTRICTIONS_UNIVERSAL,
@@ -65,47 +68,62 @@ SENSORS = {
 }
 
 
-async def async_setup_entry(hass, entry, async_add_entities):
+async def async_setup_entry(
+    hass: HomeAssistant, entry: ConfigEntry, async_add_entities: Callable
+) -> None:
     """Set up RainMachine sensors based on a config entry."""
-    rainmachine = hass.data[RAINMACHINE_DOMAIN][DATA_CLIENT][entry.entry_id]
+    controller = hass.data[DOMAIN][DATA_CONTROLLER][entry.entry_id]
+    coordinators = hass.data[DOMAIN][DATA_COORDINATOR][entry.entry_id]
+
+    @callback
+    def async_get_sensor(api_category: str) -> partial:
+        """Generate the appropriate sensor object for an API category."""
+        if api_category == DATA_PROVISION_SETTINGS:
+            return partial(
+                ProvisionSettingsSensor,
+                coordinators[DATA_PROVISION_SETTINGS],
+            )
+
+        return partial(
+            UniversalRestrictionsSensor,
+            coordinators[DATA_RESTRICTIONS_UNIVERSAL],
+        )
+
     async_add_entities(
         [
-            RainMachineSensor(
-                rainmachine,
+            async_get_sensor(api_category)(
+                controller,
                 sensor_type,
                 name,
                 icon,
                 unit,
                 device_class,
                 enabled_by_default,
-                api_category,
             )
             for (
                 sensor_type,
                 (name, icon, unit, device_class, enabled_by_default, api_category),
             ) in SENSORS.items()
-        ],
+        ]
     )
 
 
 class RainMachineSensor(RainMachineEntity):
-    """A sensor implementation for raincloud device."""
+    """Define a general RainMachine sensor."""
 
     def __init__(
         self,
-        rainmachine,
-        sensor_type,
-        name,
-        icon,
-        unit,
-        device_class,
-        enabled_by_default,
-        api_category,
-    ):
+        coordinator: DataUpdateCoordinator,
+        controller: Controller,
+        sensor_type: str,
+        name: str,
+        icon: str,
+        unit: str,
+        device_class: str,
+        enabled_by_default: bool,
+    ) -> None:
         """Initialize."""
-        super().__init__(rainmachine)
-
-        self._api_category = api_category
+        super().__init__(coordinator, controller)
         self._device_class = device_class
         self._enabled_by_default = enabled_by_default
         self._icon = icon
@@ -115,7 +133,7 @@ class RainMachineSensor(RainMachineEntity):
         self._unit = unit
 
     @property
-    def entity_registry_enabled_default(self):
+    def entity_registry_enabled_default(self) -> bool:
         """Determine whether an entity is enabled by default."""
         return self._enabled_by_default
 
@@ -132,56 +150,47 @@ class RainMachineSensor(RainMachineEntity):
     @property
     def unique_id(self) -> str:
         """Return a unique, Home Assistant friendly identifier for this entity."""
-        return "{0}_{1}".format(
-            self.rainmachine.device_mac.replace(":", ""), self._sensor_type
-        )
+        return f"{self._unique_id}_{self._sensor_type}"
 
     @property
-    def unit_of_measurement(self):
+    def unit_of_measurement(self) -> str:
         """Return the unit the value is expressed in."""
         return self._unit
 
-    async def async_added_to_hass(self):
-        """Register callbacks."""
-        self._dispatcher_handlers.append(
-            async_dispatcher_connect(self.hass, SENSOR_UPDATE_TOPIC, self._update_state)
-        )
-        await self.rainmachine.async_register_sensor_api_interest(self._api_category)
-        self.update_from_latest_data()
 
-    async def async_will_remove_from_hass(self):
-        """Disconnect dispatcher listeners and deregister API interest."""
-        super().async_will_remove_from_hass()
-        self.rainmachine.async_deregister_sensor_api_interest(self._api_category)
+class ProvisionSettingsSensor(RainMachineSensor):
+    """Define a sensor that handles provisioning data."""
 
     @callback
-    def update_from_latest_data(self):
-        """Update the sensor's state."""
+    def update_from_latest_data(self) -> None:
+        """Update the state."""
         if self._sensor_type == TYPE_FLOW_SENSOR_CLICK_M3:
-            self._state = self.rainmachine.data[DATA_PROVISION_SETTINGS]["system"].get(
+            self._state = self.coordinator.data["system"].get(
                 "flowSensorClicksPerCubicMeter"
             )
         elif self._sensor_type == TYPE_FLOW_SENSOR_CONSUMED_LITERS:
-            clicks = self.rainmachine.data[DATA_PROVISION_SETTINGS]["system"].get(
-                "flowSensorWateringClicks"
+            clicks = self.coordinator.data["system"].get("flowSensorWateringClicks")
+            clicks_per_m3 = self.coordinator.data["system"].get(
+                "flowSensorClicksPerCubicMeter"
             )
-            clicks_per_m3 = self.rainmachine.data[DATA_PROVISION_SETTINGS][
-                "system"
-            ].get("flowSensorClicksPerCubicMeter")
 
             if clicks and clicks_per_m3:
                 self._state = (clicks * 1000) / clicks_per_m3
             else:
                 self._state = None
         elif self._sensor_type == TYPE_FLOW_SENSOR_START_INDEX:
-            self._state = self.rainmachine.data[DATA_PROVISION_SETTINGS]["system"].get(
-                "flowSensorStartIndex"
-            )
+            self._state = self.coordinator.data["system"].get("flowSensorStartIndex")
         elif self._sensor_type == TYPE_FLOW_SENSOR_WATERING_CLICKS:
-            self._state = self.rainmachine.data[DATA_PROVISION_SETTINGS]["system"].get(
+            self._state = self.coordinator.data["system"].get(
                 "flowSensorWateringClicks"
             )
-        elif self._sensor_type == TYPE_FREEZE_TEMP:
-            self._state = self.rainmachine.data[DATA_RESTRICTIONS_UNIVERSAL][
-                "freezeProtectTemp"
-            ]
+
+
+class UniversalRestrictionsSensor(RainMachineSensor):
+    """Define a sensor that handles universal restrictions data."""
+
+    @callback
+    def update_from_latest_data(self) -> None:
+        """Update the state."""
+        if self._sensor_type == TYPE_FREEZE_TEMP:
+            self._state = self.coordinator.data["freezeProtectTemp"]
