@@ -1,24 +1,18 @@
 """Support for Spider switches."""
-import logging
-
 from homeassistant.components.switch import SwitchEntity
 
-from . import DOMAIN as SPIDER_DOMAIN
-
-_LOGGER = logging.getLogger(__name__)
+from .const import DOMAIN
 
 
-def setup_platform(hass, config, add_entities, discovery_info=None):
-    """Set up the Spider thermostat."""
-    if discovery_info is None:
-        return
-
-    devices = [
-        SpiderPowerPlug(hass.data[SPIDER_DOMAIN]["controller"], device)
-        for device in hass.data[SPIDER_DOMAIN]["power_plugs"]
-    ]
-
-    add_entities(devices, True)
+async def async_setup_entry(hass, config, async_add_entities):
+    """Initialize a Spider thermostat."""
+    api = hass.data[DOMAIN][config.entry_id]
+    async_add_entities(
+        [
+            SpiderPowerPlug(api, entity)
+            for entity in await hass.async_add_executor_job(api.get_power_plugs)
+        ]
+    )
 
 
 class SpiderPowerPlug(SwitchEntity):
